@@ -1,5 +1,5 @@
 /* vdesk: virtual desktops for any window manager      vim:et:ts=4:
- * Copyright 2001, 2003 Adam Sampson <azz@us-lot.org>
+ * Copyright 2001, 2003, 2025 Adam Sampson <ats@offog.org>
  * Originally based on code from aewm:
  * Copyright (c) 1998-2001 Decklin Foster.
  * 
@@ -12,13 +12,14 @@
  * provided that this copyright and notice are not removed or altered.
  */
 
+#include <stddef.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
+#include <X11/Xatom.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#include <X11/Xatom.h>
 
 Display *dpy;
 Window root;
@@ -29,21 +30,24 @@ long todesk, curdesk = 1;
 
 void set_window_desk_vis(Window w, long n, long vis) {
     long buf[2];
-    buf[0] = n; buf[1] = vis;
+    buf[0] = n;
+    buf[1] = vis;
     XChangeProperty(dpy, w, vdesk_desktop, vdesk_desktop, 32, PropModeReplace, 
         (unsigned char *)buf, 2);
 }
 
 long get_window_desk_vis(Window w, long *ovis) {
-    Atom real_type; int real_format;
+    Atom real_type;
+    int real_format;
     unsigned long items_read, items_left;
-    long *data, n = -1, vis = -1;
+    unsigned char *data;
+    long n = -1, vis = -1;
 
     if (XGetWindowProperty(dpy, w, vdesk_desktop, 0L, 2L, False,
             vdesk_desktop, &real_type, &real_format, &items_read, &items_left,
-            (unsigned char **) &data) == Success && items_read) {
-        n = data[0];
-        vis = data[1];
+            &data) == Success && items_read) {
+        n = ((long *)data)[0];
+        vis = ((long *)data)[1];
         XFree(data);
     }
 
@@ -60,14 +64,16 @@ long get_window_desk_vis(Window w, long *ovis) {
 
 long get_wm_state(Window w)
 {
-    Atom real_type; int real_format;
+    Atom real_type;
+    int real_format;
     unsigned long items_read, items_left;
-    long *data, state = WithdrawnState;
+    unsigned char *data;
+    long state = WithdrawnState;
 
     if (XGetWindowProperty(dpy, w, wm_state, 0L, 2L, False,
             wm_state, &real_type, &real_format, &items_read, &items_left,
-            (unsigned char **) &data) == Success && items_read) {
-        state = *data;
+            &data) == Success && items_read) {
+        state = ((long *)data)[0];
         XFree(data);
     }
     return state;
@@ -169,7 +175,7 @@ int main(int argc, char **argv) {
     if (optind == argc) {
         /* Print the current desktop number. */
         printf("%ld\n", curdesk);
-        return 0;
+        goto out;
     }
 
     if ((argc - optind) == 1)
@@ -212,6 +218,7 @@ int main(int argc, char **argv) {
         }
     }
 
-    XSync(dpy, False);
+out:
+    XCloseDisplay(dpy);
     return 0;
 }
